/*********************************************************************
 * Job Ticket API (JTAPI)                                            
 *                                                                   
 * Copyright (c) 2003 International Business Machines Corp.          
 * Copyright (c) 2003 Free Standards Group.          
 *                                                                    
 *                    Common Public License - v 1.0 
 * THE ACCOMPANYING PROGRAM IS PROVIDED UNDER THE TERMS OF THIS COMMON 
 * PUBLIC LICENSE ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION 
 * OF THE PROGRAM CONSTITUTES RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT.   
 *
 * The Common Public License in its entirety is located in 
 * jtapi_cpl.h and applies to the contents of this file. 
 *********************************************************************/

#ifndef JTAPI_ATTRIBUTE_H
#define JTAPI_ATTRIBUTE_H

#include <jtapi.h>
#include <jtapi_enums.h>

/************************************************************
 * newAttribute - Creates a new Attribute object having the
 * provided attribute name, value type, and value.
 *
 * param jtapi_attribute_t* - Pointer to the new attribute
 * param char* attrName - Name for the new attribute
 * param char* valueType - Type for the new attribute
 * param void* value - Pointer to the value for this attribute.
 *      Multiple values should be added with the add() 
 *      function.
 *
 * return jtapi_status_t - JTAPI_OK on success
 ************************************************************/
jtapi_status_t newAttribute(jtapi_attribute_t* attribute, 
                            char* attrName, 
                            char* valueType, 
                            void* value);

/************************************************************
 * destroyAttribute - Free the memory used by the Attribute.
 *
 * param jtapi_attribute_t attribute - Attribute to free
 ************************************************************/
void destroyAttribute(jtapi_attribute_t attribute);

/************************************************************
 * add - Add an additional value to this Attribute.
 *
 * param jtapi_attribute_t attribute - Attribute 
 * param void* value - Pointer to value to add
 *
 * return jtapi_status_t - JTAPI_OK on success
 ************************************************************/
jtapi_status_t addValue(jtapi_attribute_t attribute, void* value);

/************************************************************
 * attributesEqual - Checks the equality of the provided 
 * Attributes. All data members of the Attribute must be 
 * equal for the Attributes to be equal.
 *
 * param jtapi_attribute_t attr1  
 * param jtapi_attribute_t attr2  
 *
 * return jtapi_boolean_t - true if the Attributes are equal
 ************************************************************/
jtapi_boolean_t attributesEqual(jtapi_attribute_t attr1,
                                jtapi_attribute_t attr2);

/************************************************************
 * getName - Get the name of the Attribute.
 *
 * param jtapi_attribute_t attribute - Attribute 
 *
 * return char* - Name of the attribute
 ************************************************************/
char* getName(jtapi_attribute_t attribute);

/************************************************************
 * getNumValues - Returns the number of values that the 
 * Attribute contains.
 *
 * param jtapi_attribute_t attribute - Attribute 
 *
 * return int - Number of values
 ************************************************************/
int getNumValues(jtapi_attribute_t attribute);

/************************************************************
 * getValueType - Returns the type of the Attribute's values.
 *
 * param jtapi_attribute_t attribute - Attribute 
 *
 * return char* - Type of this Attribute'e values
 ************************************************************/
char* getValueType(jtapi_attribute_t attribute);

/************************************************************
 * hasNextValue - Used to iterate over the Attribute's 
 * values. Returns true while there are more values.
 *
 * param jtapi_attribute_t attribute - Attribute 
 *
 * return jtapi_boolean_t - true while there are more 
 *        values
 ************************************************************/
jtapi_boolean_t hasNextValue(jtapi_attribute_t attribute);

/************************************************************
 * nextValue - Used to iterate over the Attribute's values. 
 * Returns a pointer to the next value.
 *
 * param jtapi_attribute_t attribute - Attribute 
 *
 * return void* - Pointer to the next value
 ************************************************************/
void* nextValue(jtapi_attribute_t attribute);

/************************************************************
 * resetIterator - Reset the iterator.
 *
 * param jtapi_attribute_t attribute - Attribute 
 ************************************************************/
void resetIterator(jtapi_attribute_t attr);

#endif /* JTAPI_ATTRIBUTE_H */
